import datetime


class Request:

    def __init__(self):
        self.function = None
        self.equipments = None
        self.start = None
        self.end = None
        self.aggregation = None
        self.interval = None
        self.filters = None
        self.on_off_sensor = None
        self.restart_time = None
        self.sensitivity = None
        self.dataframe = None
        self.extra_data = None
        self.target_feat = None
        self.connections = None
        self.name = None

    def prepare(self):
        query = {}
        if self.equipments is not None:
            query["equipments_list"] = self.equipments
        if self.start is not None and self.end is not None:
            query["timeframe"] = {
                "start": self.__format_date(self.start),
                "end": self.__format_date(self.end)
            }
        return query

    def __format_date(self, dt_to_format):
        if isinstance(dt_to_format, datetime.datetime):
            millisec = dt_to_format.timestamp() * 1000
            return int(millisec)




import requests
from ..wizata_dsapi_entities import request

class WizataDSAPIClient:

    def __init__(self):
        self.domain = None
        self.user = None
        self.password = None

    def __url(self):
        return "https://" + self.user + ":" + self.password + "@" + self.domain + "/dsapi/"

    def __header(self):
        return {'Content-Type': 'application/json'}

    def __request_process(self, method, route):
        response = requests.request(method, self.__url() + route, headers=self.__header())
        if response.status_code == 200:
            return response.json()
        else:
            raise RuntimeError(str(response.status_code) + " - " + response.reason)

    def get_infos(self):
        return self.__request_process("GET", "get_infos")

    def get_ds_functions(self):
        return self.__request_process("GET", "get_ds_functions")

    def get_datas(self, query: request):
        return query.prepare()



from .mlmodel import MLModel
from .plot import Plot
from .request import Request


class Execution:

    def __init__(self, experiment_id=None, experiment_type=None, request: Request = None):
        self.experiment_id = experiment_id

        # Experiment Type = "function" or "anomaly_detection"
        self.experiment_type = experiment_type

        # Execution = "wizard" automl or "simulate" execute trained wizatads
        self.execution_type = "wizard"

        self.request = request
        self.dataframe = None
        self.models = []
        self.anomalies = []
        self.plots = []

        self.result_dataframe = None

    def append_plot(self, figure, name="Unkwown"):
        plot = Plot(name)
        plot.experiment_id = self.experiment_id
        plot.figure = figure
        self.plots.append(plot)
        return plot

    def append_model(self, trained_model, input_columns, output_columns=None, has_anomalies=False, scaler=None):
        ml_model = MLModel()

        ml_model.trained_model = trained_model
        ml_model.scaler = scaler

        ml_model.input_columns = input_columns
        ml_model.output_columns = output_columns

        ml_model.has_anomalies = has_anomalies

        self.models.append(ml_model)
        return ml_model



from setuptools import setup

setup(
    name='wizata-dsapi',
    version='0.0.10',
    description='Wizata Data Science Toolkit',
    author='Wizata S.A.',
    author_email='info@wizata.com',
    packages=['wizata_dsapi_entities','wizata_dsapi_tools'],
)
